#!/usr/bin/env bash

# Note: this runs as-is, pretty much without external
# dependencies. The OpenWrt ImageBuilder contains the toolchain and
# everything that is needed to build the firmware images.

set -e

TARGET_ARCHITECTURE=$1
TARGET_VARIANT=$2
TARGET_DEVICE=$3

BUILD="$(dirname "${0}")/build/"
BUILD="$(readlink -f "${BUILD}")"

###
### chose a release or "SNAPSHOT"
###
RELEASE="23.05.4"

if [ RELEASE = "SNAPSHOT" ]; then
    IMGBUILDER_NAME="openwrt-imagebuilder-${TARGET_ARCHITECTURE}-${TARGET_VARIANT}.Linux-x86_64"
    IMGBUILDER_ARCHIVE="${IMGBUILDER_NAME}.tar.zst"
    HOME_URL="/home/ataraxia/projects/openwrt-snapshot/r27082-b733b6acb5"
    IMGBUILDERURL="${HOME_URL}/snapshots/targets/${TARGET_ARCHITECTURE}/${TARGET_VARIANT}/${IMGBUILDER_ARCHIVE}"
else
    IMGBUILDER_NAME="openwrt-imagebuilder-${RELEASE}-${TARGET_ARCHITECTURE}-${TARGET_VARIANT}.Linux-x86_64"
    IMGBUILDER_ARCHIVE="${IMGBUILDER_NAME}.tar.xz"
    HOME_URL=""
    IMGBUILDERURL="https://downloads.openwrt.org/releases/${RELEASE}/targets/${TARGET_ARCHITECTURE}/${TARGET_VARIANT}/${IMGBUILDER_ARCHIVE}"
fi

IMGBUILDER_DIR="${BUILD}/${IMGBUILDER_NAME}"
IMGTEMPDIR="${BUILD}/image-extras"


if [ -z ${TARGET_DEVICE} ]; then
    echo "Usage: $0 architecture variant device-profile"
    echo " e.g.: $0 ath79 generic tplink_tl-wr1043nd-v1"
    echo "       $0 ath79 generic tplink_archer-c6-v2"
    echo "       $0 ath79 generic tplink_tl-wdr4300-v1"
    echo "       $0 ipq40xx mikrotik mikrotik_hap-ac2"
    echo " to get a list of supported devices issue a 'make info' in the OpenWRT image builder directory:"
    echo "   '${IMGBUILDER_DIR}'"
    echo " the build results will be under '${IMGBUILDER_DIR}/bin/targets/'"
    kill -INT $$
fi

# the absolute minimum for extroot to work at all (i.e. when the disk is already set up, for example by hand).
PREINSTALLED_PACKAGES="block-mount kmod-fs-f2fs kmod-usb-storage"

# these are needed for the proper functioning of the auto extroot scripts
# blockdev is needed to re-read the partition table using `blockdev --rereadpt /dev/sdX`
PREINSTALLED_PACKAGES+=" mount-utils swap-utils sgdisk blockdev f2fs-tools"

# the following packages are optional, feel free to (un)comment them
PREINSTALLED_PACKAGES+=" zram-swap kmod-lib-lz4 logrotate dnsmasq-full"

# you exclude packages with this to shrink the image for routers with smaller flash storage.
SAVE_SPACE_PACKAGES=" -ppp -ppp-mod-pppoe -dnsmasq -luci"

PREINSTALLED_PACKAGES+=${SAVE_SPACE_PACKAGES}

# these packages would be autoprovisioned in stage2
# LuCi + some utilities
AUTOPROVISIONED_PACKAGES="luci rsync bottom drill luci-proto-wireguard kmod-nf-nathelper-extra kmod-ipt-conntrack-extra"
# AUTOPROVISIONED_PACKAGES="lua luci rsync diffutils bottom drill"
# # Wireguard + Proxies + PBR
# AUTOPROVISIONED_PACKAGES+=" luci-proto-wireguard sing-box xray-core dnscrypt-proxy2 pbr luci-app-pbr"
# # PPTP
# AUTOPROVISIONED_PACKAGES+=" ppp-mod-pptp kmod-pptp kmod-nf-nathelper-extra kmod-ipt-conntrack-extra luci-proto-ppp"

mkdir -pv "${BUILD}"

rm -rf "${IMGTEMPDIR}"
cp -r image-extras/common/ "${IMGTEMPDIR}"

# Setup root password
if [ -f .rootpass ]; then
    rootpass=$(cat .rootpass)
    sed -i "s/ROOT_PASS/\"${rootpass}\"/" "${IMGTEMPDIR}/root/autoprovision-stage2.sh"
else
    sed -i "s/ROOT_PASS/\"\"/" "${IMGTEMPDIR}/root/autoprovision-stage2.sh"
fi
# Setup autoprovisioned pkgs
sed -i "s/AUTOPROVISIONED_PACKAGES/${AUTOPROVISIONED_PACKAGES}/" "${IMGTEMPDIR}/root/autoprovision-stage2.sh"

PER_PLATFORM_IMAGE_EXTRAS="image-extras/${TARGET_DEVICE}/"
if [ -e "${PER_PLATFORM_IMAGE_EXTRAS}" ]; then
    rsync -pr "${PER_PLATFORM_IMAGE_EXTRAS}" "${IMGTEMPDIR}/"
fi

if [ ! -e "${IMGBUILDER_DIR}" ]; then
    pushd "${BUILD}"
    # --no-check-certificate if needed
    wget --continue "${IMGBUILDERURL}"
    xz -d <"${IMGBUILDER_ARCHIVE}" | tar vx

    # cp "${IMGBUILDERURL}" ./
    # zstd -d "${IMGBUILDER_ARCHIVE}" --stdout | tar xvf -
    # sed -i "s|https://downloads.openwrt.org|file://${HOME_URL}|" "${IMGBUILDER_DIR}/repositories.conf"
    popd
fi

pushd "${IMGBUILDER_DIR}"

make image PROFILE=${TARGET_DEVICE} PACKAGES="${PREINSTALLED_PACKAGES}" FILES=${IMGTEMPDIR}

pushd "bin/targets/${TARGET_ARCHITECTURE}/"
ln -sf ../../../packages .
popd

popd
