#!/bin/sh

# autoprovision stage 2: this script will be executed upon boot if the extroot was successfully mounted (i.e. rc.local is run from the extroot overlay)

. /root/autoprovision-functions.sh

installPackages()
{
    signalAutoprovisionWaitingForUser

    until (opkg update)
     do
        log "opkg update failed. No internet connection? Retrying in 15 seconds..."
        sleep 15
        # Initiate a synchronous time update.
        ntpd -d -q -n -p openwrt.pool.ntp.org
    done

    signalAutoprovisionWorking

    log "Autoprovisioning stage2 is about to install packages"

    # switch ssh from dropbear to openssh (needed to install sshtunnel)
    /etc/init.d/dropbear stop
    opkg remove dropbear
    rm -rf /etc/dropbear

    opkg install openssh-server openssh-sftp-server sshtunnel

    sed -i 's/^#PermitRootLogin prohibit-password/PermitRootLogin yes/' /etc/ssh/sshd_config
    chmod 0700 /root/.ssh
    /etc/init.d/sshd enable
    /etc/init.d/sshd restart

    # CUSTOMIZE
    # install some more packages that don't need any extra steps
    opkg install AUTOPROVISIONED_PACKAGES

    # just in case if we were run in a firmware that didn't already have luci
    /etc/init.d/uhttpd enable
}

autoprovisionStage2()
{
    log "Autoprovisioning stage2 speaking"

    # TODO this is a rather sloppy way to test whether stage2 has been done already, but this is a shell script...
    if [ $(uci get system.@system[0].log_type) == "file" ]; then
        log "Seems like autoprovisioning stage2 has been done already. Running stage3."
        #/root/autoprovision-stage3.py
    else
        signalAutoprovisionWorking

        log "Starting ntpd to update system time; otherwise the openwrt.org certificates are rejected as not yet valid."
        # Added -l hoping that it may help against ntpd quitting.
        ntpd -l -N -p openwrt.pool.ntp.org

        # CUSTOMIZE: with an empty argument it will set a random password and only ssh key based login will work.
        # please note that stage2 requires internet connection to install packages and you most probably want to log in
        # on the GUI to set up a WAN connection. but on the other hand you don't want to end up using a publically
        # available default password anywhere, therefore the random here...
        setRootPassword ROOT_PASS

        # Setup stangri's repo
        echo -e -n 'untrusted comment: OpenWrt usign key of Stan Grishin\nRWR//HUXxMwMVnx7fESOKO7x8XoW4/dRidJPjt91hAAU2L59mYvHy0Fa\n' > /etc/opkg/keys/7ffc7517c4cc0c56
        sed -i '/stangri_repo/d' /etc/opkg/customfeeds.conf
        echo 'src/gz stangri_repo https://repo.openwrt.melmac.net' >> /etc/opkg/customfeeds.conf

        # Preserve opkg list
        sed -i -e "/^lists_dir\s/s:/var/opkg-lists$:/usr/lib/opkg/lists:" /etc/opkg.conf

        mkdir -p /var/log/archive
        # logrotate is complaining without this directory
        mkdir -p /var/lib

        uci set system.@system[0].log_type=file
        uci set system.@system[0].log_file=/var/log/syslog
        uci set system.@system[0].log_size=0
        uci set system.@system[0].zram_comp_algo="lz4"
        # uci set system.@system[0].zram_size_mb="64"
        uci set network.lan.ipaddr="10.10.10.1"
        uci commit
        /etc/init.d/network restart

        installPackages

        crontab - <<EOF
# */10 * * * * /root/autoprovision-stage3.py
0 0 * * * /usr/sbin/logrotate /etc/logrotate.conf
EOF

        sync
        reboot
    fi
}

autoprovisionStage2
